@@ -10,21 +10,37 @@ from django.core.management.base import LabelCommand, BaseCommand
 from optparse import make_option
 from django.db import models
 
+INTEGER = ['BigIntegerField', 'IntegerField', 'AutoField',
+           'PositiveIntegerField', 'PositiveSmallIntegerField']
+FLOAT = ['DecimalField', 'FloatField']
+NUMERIC = INTEGER + FLOAT
 # Note if mappings are manually specified they are of the following form ...
 # MAPPINGS = "column1=shared_code,column2=org(Organisation|name),column3=description"
 # statements = re.compile(r";[ \t]*$", re.M)
 
 def save_csvimport(props=None, instance=None):
     """ To avoid circular imports do saves here """
-    if not instance:
-        from csvimport.models import CSVImport
-        csvimport = CSVImport()
-    if props:
-        for key, value in props.items():
-            csvimport.__setattr__(key, value)
-    csvimport.save()
-    return csvimport.id
-
+    try:
+        if not instance:
+            from csvimport.models import CSVImport
+            csvimp = CSVImport()
+        if props:
+            for key, value in props.items():
+                csvimp.__setattr__(key, value)
+        csvimp.save()
+        return csvimp.id
+    except:
+        # Running as command line
+        print 'Assumed charset = %s\n' % instance.charset
+        print '###############################\n' 
+        for line in instance.loglist:
+            if type(line) != type(''):
+                for subline in line:
+                    print line
+                    print
+            else:
+                print line
+                print
 
 class Command(LabelCommand):
     """
@@ -39,6 +55,8 @@ class Command(LabelCommand):
                            help='Please provide the file to import from'),
                make_option('--model', default='iisharing.Item', 
                            help='Please provide the model to import to'),
+               make_option('--charset', default='', 
+                           help='Force the charset conversion used rather than detect it')
                    )
     help = "Imports a CSV file to a model"
 
@@ -58,6 +76,7 @@ class Command(LabelCommand):
         self.nameindexes = False
         self.deduplicate = True
         self.csvfile = []
+        self.charset = ''
 
     def handle_label(self, label, **options):
         """ Handle the circular reference by passing the nested
@@ -66,20 +85,29 @@ class Command(LabelCommand):
         filename = label 
         mappings = options.get('mappings', []) 
         modelname = options.get('model', 'Item')
+        charset = options.get('charset','')
         # show_traceback = options.get('traceback', True)
-        self.setup(mappings, modelname, filename)
+        self.setup(mappings, modelname, charset, filename)
+        if not hasattr(self.model, '_meta'):
+            msg = 'Sorry your model could not be found please check app_label.modelname'
+            try:
+                print msg
+            except:
+                self.loglist.append(msg)
+            return
         errors = self.run()
         if self.props:
-            save_csvimport(self.props)
-        raise Exception(errors)
+            save_csvimport(self.props, self)
+        self.loglist.extend(errors)
         return
 
-    def setup(self, mappings, modelname, csvfile='', defaults='',
+    def setup(self, mappings, modelname, charset, csvfile='', defaults='',
               uploaded=None, nameindexes=False, deduplicate=True):
         """ Setup up the attributes for running the import """
         self.defaults = self.__mappings(defaults)
         if modelname.find('.') > -1:
             app_label, model = modelname.split('.')
+        self.charset = charset
         self.app_label = app_label
         self.model = models.get_model(app_label, model)
         if mappings:
@@ -127,12 +155,13 @@ class Command(LabelCommand):
         else:
             csvimportid = 0
         mapping = []
+        fieldmap = {}
+        for field in self.model._meta.fields:
+            fieldmap[field.name] = field
+
         if self.mappings:
-             self.loglist.append('Using manually entered mapping list') 
+            self.loglist.append('Using manually entered mapping list') 
         else:
-            fieldmap = {}
-            for field in self.model._meta.fields:
-                fieldmap[field.name] = field
             for i, heading in enumerate(self.csvfile[0]):
                 for key in heading, heading.lower():
                     if fieldmap.has_key(key):
@@ -154,6 +183,7 @@ class Command(LabelCommand):
             model_instance = self.model()
             model_instance.csvimport_id = csvimportid
             for (column, field, foreignkey) in self.mappings:
+                field_type = fieldmap.get(field).get_internal_type()
                 if self.nameindexes:
                     column = indexes.index(column)
                 else:
@@ -167,8 +197,22 @@ class Command(LabelCommand):
                 if self.debug:
                     self.loglist.append('%s.%s = "%s"' % (self.model.__name__, 
                                                           field, row[column]))
-                #if type(row[column]) == type(''):
-                #    row[column] = unicode(row[column])
+                # Tidy up numeric data    
+                if field_type in NUMERIC:
+                    if not row[column]:
+                        row[column] = 0
+                    elif field_type in INTEGER:
+                        row[column] = float(row[column])
+                        if row[column] > 9223372036854775807:
+                            self.loglist.append('Column %s = %s more than the max integer 9223372036854775807' \
+                                                % (field, row[column]))
+                        row[column] = int(row[column])
+                        if row[column] < 0 and field_type.startswith('Positive'):
+                            self.loglist.append('Column %s = %s, less than zero so set to 0' \
+                                                % (field, row[column]))
+                            row[column] = 0
+                    else:
+                        row[column] = float(row[column])
                 try:
                     model_instance.__setattr__(field, row[column])
                 except:
@@ -256,10 +300,11 @@ class Command(LabelCommand):
     def __csvfile(self, datafile):
         """ Detect file encoding and open appropriately """
         filehandle = open(datafile)
-        diagnose = chardet.detect(filehandle.read())
-        charset = diagnose['encoding']
+        if not self.charset:
+            diagnose = chardet.detect(filehandle.read())
+            self.charset = diagnose['encoding']
         try:
-            csvfile = codecs.open(datafile, 'r', charset)
+            csvfile = codecs.open(datafile, 'r', self.charset)
         except IOError:
             self.error('Could not open specified csv file, %s, or it does not exist' % datafile, 0)
         else:
@@ -267,7 +312,7 @@ class Command(LabelCommand):
             # perform list commands and since list is an acceptable iterable, 
             # we'll just transform it.
             return list(self.charset_csv_reader(csv_data=csvfile, 
-                                                charset=charset))
+                                                charset=self.charset))
 
     def charset_csv_reader(self, csv_data, dialect=csv.excel, 
                            charset='utf-8', **kwargs):
